@@ -202,13 +202,19 @@ class Aggregate:
             folders = list(df_curves_info['folder'])
             pys = list(df_curves_info['py'])
             paths = list(df_curves_info['path'])
-
-            # df.to_csv('df.csv',index=False)
+            para_combination = list(df_curves_info['para_combination'])
 
             # Aggregate Equity
             dfs = []
             for i, path in enumerate(paths):
-                df = pd.read_csv(path, index_col='date')
+                intraday = para_combination[i]['intraday']
+                summary = para_combination[i]['summary_mode']
+                df_csv = pd.read_csv(path, index_col='date')
+                if (intraday or summary):
+                    df = plotguy.resample_summary_to_daily(para_combination=para_combination[i],folder=folders[i])
+                else:
+                    df = df_csv
+
                 df_current = pd.DataFrame()
                 if normalized_boolean:
                     df_current[f'{folders[i]}_{pys[i]}_{i}'] = df['equity_value'] / (df['equity_value'].iloc[0] / 100000)
@@ -248,10 +254,12 @@ class Aggregate:
             for element in df_performance.columns:
                 total_dict[element] = df_performance[element].sum()
 
+
             total_dict['return_on_capital'] = total_dict['net_profit']/total_dict['initial_capital']
             total_dict['win_rate'] = total_dict['num_of_win']/total_dict['num_of_trade']
 
 
+
             df_agg = df_all_curves['Aggregate_Equity']
             dds = []
             dd_pct = []
@@ -294,6 +302,7 @@ class Aggregate:
 
             aggregate_performance = components.aggregate_performance(total_dict, year_list)
 
+
             # Generate Chart
             if aggregate_boolean:
                 fig_line = px.line()
