@@ -2,6 +2,8 @@ import pandas as pd
 import numpy as np
 import datetime
 import math
+import json
+import ast
 import os
 
 from dash import dcc, html    ## pip install dash
@@ -10,7 +12,8 @@ import plotly.express as px
 import plotly.graph_objs as go
 from plotly.subplots import make_subplots
 
-import dash_dangerously_set_inner_html
+import plotguy
+import dash_dangerously_set_inner_html   # for all Plot Class
 
 
 class Components:
@@ -291,8 +294,9 @@ class Components:
 
         for i in range(len(year_list)):
             try:
-                win_rate = "{:.0%}".format( int(df[f'{year_list[i]}_win_rate'])/100 )
-            except:
+                win_rate = "{:.0%}".format( int( float(df[f'{year_list[i]}_win_rate']) )/100 )
+            except Exception as e:
+                # print(e)
                 win_rate = '-----'
 
             year_col1.append(dbc.Row([dbc.Col(year_list[i],style={'margin-left':'15px'} ,width=3),
@@ -349,17 +353,6 @@ class Components:
         return matrix_div
 
 
-    def selection_title(self, para_dict, values):
-        title = []
-        for i, key in enumerate(para_dict):
-            title.append(dbc.Row(
-                [dbc.Col(key, width=6),
-                 dbc.Col(f'{values[i]}', style={'text-align': 'center'}, width=6)]))
-
-        return title
-
-
-
     filter_options = {
         'num_of_trade':'Trade Count',
         'return_on_capital': 'Return on Capital',
@@ -407,6 +400,7 @@ class Components:
 
         df_sorted = df_sorted.reset_index(drop=True)
 
+
         line_colour = []
         for c in range(len(df_sorted)):
             profile = c % 6
@@ -426,11 +420,91 @@ class Components:
         elif profile == 5:  rgb = (int(252 * degree), 0, 252)
         return 'rgb' + str(rgb)
 
+    def generate_chart_3(self, para_combination, chart3_start, chart3_end, freq):
+
+        data_folder = para_combination['data_folder']
+        code = para_combination['code']
+        data_path = os.path.join(data_folder, code + '.csv')
+        df = pd.read_csv(data_path)
+
+        df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
+        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y-%m-%d %H:%M:%S')
+        df = df.loc[(df['date'] >= chart3_start) & (df['date'] <= chart3_end)]
+
+        # grab first and last observations from df.date and make a continuous date range from that
+        dt_all = pd.date_range(start=df['datetime'].iloc[0], end=df['datetime'].iloc[-1], freq=freq)
+        # check which dates from your source that also accur in the continuous date range
+        dt_obs = [d.strftime("%Y-%m-%d %H:%M:%S") for d in df['datetime']]
+        # isolate missing timestamps
+        dt_breaks = [d for d in dt_all.strftime("%Y-%m-%d %H:%M:%S").tolist() if not d in dt_obs]
+
+        save_path = plotguy.generate_filepath(para_combination)
+        df_signal_list = pd.read_csv(save_path)
+        df_signal_list['date'] = pd.to_datetime(df_signal_list['date'], format='%Y-%m-%d')
+        df_signal_list['datetime'] = pd.to_datetime(df_signal_list['datetime'], format='%Y-%m-%d %H:%M:%S')
+        df_signal_list = df_signal_list.loc[
+            (df_signal_list['date'] >= chart3_start) & (df_signal_list['date'] <= chart3_end)]
+
+        df_open = df_signal_list.loc[df_signal_list['action'] == 'open']
+        df_profit = df_signal_list.loc[df_signal_list['action'] == 'profit_target']
+        df_stop_loss = df_signal_list.loc[df_signal_list['action'] == 'stop_loss']
+        df_close = df_signal_list.loc[df_signal_list['action'] == 'close_logic']
+
+        fig = go.Figure(data=[go.Candlestick(x=df['datetime'],
+                                             open=df['open'],
+                                             high=df['high'],
+                                             low=df['low'],
+                                             close=df['close'],
+                                             increasing_line_color='white')
+                              ])
+
+        fig.update_layout(title = {'text': 'Candlestick Chart' , 'font': {'size': 12 }  })
+
+        fig.update_xaxes(rangebreaks=[dict(dvalue=15 * 60 * 1000, values=dt_breaks)])
+
+        fig.add_trace(go.Scatter(mode='markers',
+                                 x=df_open['datetime'], y=df_open['close'],  # visible='legendonly',
+                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
+                                             line=dict(color='yellow', width=2.5)), name='Open'))
+        fig.add_trace(go.Scatter(mode='markers',
+                                 x=df_profit['datetime'], y=df_profit['close'],  # visible='legendonly',
+                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
+                                             line=dict(color='Cyan', width=2.5)), name='Profit Target'))
+        fig.add_trace(go.Scatter(mode='markers',
+                                 x=df_stop_loss['datetime'], y=df_stop_loss['close'],  # visible='legendonly',
+                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
+                                             line=dict(color='Cyan', width=2.5)), name='Stop Loss'))
+        fig.add_trace(go.Scatter(mode='markers',
+                                 x=df_close['datetime'], y=df_close['close'],  # visible='legendonly',
+                                 marker=dict(color='rgba(0, 0, 0, 0)', size=18,
+                                             line=dict(color='Cyan', width=2.5)), name='Close Logic'))
+
+        fig.update_layout(xaxis_rangeslider_visible=False)
+
+        chart_bg = '#1f2c56'
+        fig.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=500,
+                          margin=dict(l=85, r=25, t=60, b=0),
+                          showlegend=False,
+                          font={"color": "white", 'size': 10.5}, yaxis={'title': 'Equity'},
+                          xaxis={'title': ''}
+                          )
+        fig.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
+        fig.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
+
+
+
+
+        return fig
+
+
+    def generate_chart_1(self, graph_df, all_para_combination, intraday, summary_mode):
+        py_filename = all_para_combination[0]['py_filename']
+        para_dict = all_para_combination[0]['para_dict']
+        para_key_list = list(para_dict)
+        df_all_para_combination = pd.DataFrame(all_para_combination)
 
-    def generate_chart_1(self, graph_df, para_key_list, generate_filepath, filename, output_folder, start_date, end_date,
-                   para_dict):
         fig_line = px.line()
-        fig_line.update_layout(title = {'text': filename})
+        fig_line.update_layout(title = {'text': py_filename})
         fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
         fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
         fig_line.update_layout(plot_bgcolor=self.chart_bg, paper_bgcolor=self.chart_bg, height=500,
@@ -441,10 +515,11 @@ class Components:
                                )
 
         for i in graph_df.index:
-            para_values = []
+            # print(df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict())
+            para_combination = {}
             hovertemplate = "%{x}<br>"
             for key in para_key_list:
-                para_values.append(graph_df.iloc[i][key])
+                para_combination[key] = graph_df.iloc[i][key]
                 hovertemplate = hovertemplate + \
                                 key + " : " + str(graph_df.iloc[i][key]) + "<br>"
             hovertemplate = hovertemplate + "<br>"
@@ -457,20 +532,44 @@ class Components:
                 graph_df.iloc[i]['mdd_pct'] / 100) + "<br>"
             hovertemplate = hovertemplate + "Trade Count : " + str(graph_df.iloc[i]['num_of_trade']) + "<br>"
             hovertemplate = hovertemplate + "COV : " + str(round(graph_df.iloc[i]['cov'], 2)) + "<br>"
-            hovertemplate = hovertemplate + "Win Rate : " + "{:.0%}".format(
-                graph_df.iloc[i]['win_rate'] / 100) + "<br>"
+            try:
+                hovertemplate = hovertemplate + "Win Rate : " + "{:.0%}".format(
+                    float(graph_df.iloc[i]['win_rate']) / 100) + "<br>"
+            except:
+                hovertemplate = hovertemplate + "Win Rate : --" + "<br>"
             hovertemplate = hovertemplate + "<br>"
 
-            save_path = generate_filepath(filename, output_folder, start_date, end_date, para_dict, para_values)
+            # para_combination for save_path generation
+            reference_index = graph_df.iloc[i].reference_index
+            para_combination = df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict('records')[0]
+
             line_colour = graph_df.loc[i].line_colour
 
-            df = pd.read_csv(save_path)
-            df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
+
+
+            # df = pd.read_csv(save_path)
+            # df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
+
+            if (intraday or summary_mode):
+                df_daily = plotguy.resample_summary_to_daily(para_combination=para_combination)
+            else:
+                save_path = plotguy.generate_filepath(para_combination)
+                df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+                # df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
+                # df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
+                # df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
+                df_daily = df_backtest.copy()
+
+            start_date = para_combination['start_date']
+            end_date = para_combination['end_date']
+            df_daily['date'] = pd.to_datetime(df_daily['date'], format='%Y-%m-%d')
+            df_daily = df_daily.loc[(df_daily['date'] >= start_date) & (df_daily['date'] <= end_date)]
+            df_daily = df_daily.reset_index(drop=True)  # Reset Index
 
             hovertemplate = hovertemplate + "Equity : %{y:,.0f}"
 
             fig_line.add_trace(go.Scatter(mode='lines', hovertemplate=hovertemplate,
-                                          x=df['date'], y=df['equity_value'],
+                                          x=df_daily['date'], y=df_daily['equity_value'],
                                           line=dict(color=line_colour, width=1.5), name=''), )
         return fig_line
 
@@ -522,8 +621,68 @@ class Components:
 
         return df_chart
 
+    def generate_chart_2_summary(self, para_combination, line_colour):
+        df_daily = plotguy.resample_summary_to_daily(para_combination=para_combination)
+
+        title = ''
+        para_dict = para_combination['para_dict']
+        for key in para_dict:
+            title = title + f'{key}:{para_combination[key]} '
+
+        start_date = para_combination['start_date']
+        end_date = para_combination['end_date']
+        df_daily['date'] = pd.to_datetime(df_daily['date'], format='%Y-%m-%d')
+        df_daily = df_daily.loc[(df_daily['date'] >= start_date) & (df_daily['date'] <= end_date)]
+        df_daily = df_daily.reset_index(drop=True)  # Reset Index
+        df_chart = df_daily.copy()
+
+        row_height = [0.85, 0.15]
+        fig_line = make_subplots(rows=2, cols=1,
+                                 row_heights=row_height, shared_xaxes=True)
+
+
+        # fig_line = px.line()
+        fig_line.update_layout(title={'text': title, 'font': {'size': 12}})
+        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
+                                      x=df_chart['date'], y=df_chart['equity_value'],
+                                      line=dict(color=line_colour, width=1), name='Strategy Equity'))
+        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
+                                      x=df_chart['date'], y=df_chart['bah'],
+                                      line=dict(color='Grey', width=1), name='BnH Equity'))
+
+        hovertemplate = "Date : %{x}"
+        fig_line.add_trace(go.Scatter(mode='markers', hovertemplate=hovertemplate,
+                                      x=df_chart['date'], y=df_chart['signal_value'], # visible='legendonly',
+                                      marker=dict(color='rgba(0, 0, 0, 0)', size=9,
+                                                  line=dict(color='yellow', width=2.5)), name='Open'))
+
+        chart_bg = '#1f2c56'
+        fig_line.update_layout(plot_bgcolor=chart_bg, paper_bgcolor=chart_bg, height=500,
+                               margin=dict(l=85, r=25, t=60, b=0),
+                               # showlegend=False,
+                               font={"color": "white", 'size': 10.5}, yaxis={'title': 'Equity'},
+                               xaxis={'title': ''}
+                               )
+        fig_line.update_xaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
+        fig_line.update_yaxes(showline=True, zeroline=False, linecolor='#979A9A', gridcolor=chart_bg)
+
+        fig = self.generate_subchart(df_chart, 'volume', 'line', 'yellow')
+        fig_line.add_trace(fig, row=2, col=1)
+
+
+        return df_daily, fig_line
+
+
+
+
+        pass
+
+
+    def generate_chart_2_full(self, para_combination, line_colour, settings):
+        save_path = plotguy.generate_filepath(para_combination)
+        df_csv = pd.read_csv(save_path)
+        df_chart = self.prepare_df_chart(df_csv)
 
-    def generate_chart_2(self,df_chart,line_colour, para_dict,para_values, settings):
         df_signal = df_chart.copy()
         df_signal = df_signal.dropna(subset=['action'])
         df_signal = df_signal.reset_index()
@@ -533,9 +692,8 @@ class Components:
         signal_close_close = []
         signal_close_reason = []
 
-
         for i, row in df_signal.iterrows():
-            if row['action'] == 'open':
+            if (row['action'] == 'open') and not ( (i+1) == len(df_signal)):
                 signal_open_date.append(df_signal.iloc[i]['date'])
                 signal_open_close.append(df_signal.iloc[i]['close'])
                 signal_close_date.append(df_signal.iloc[i + 1]['date'])
@@ -582,6 +740,7 @@ class Components:
             except:
                 close_reason.append(None)
 
+
         df_signal_final = pd.DataFrame()
         df_signal_final['open_date'] = open_date
         df_signal_final['open_close'] = open_close
@@ -594,6 +753,8 @@ class Components:
         df_signal_final.close_close = df_signal_final.close_close.round(2)
         df_signal_final.pctchange = df_signal_final.pctchange.round(2)
 
+        # print(df_signal_final)
+
         hover_template = "<br>".join([
             "Close Reason: %{customdata[4]}",
             "Open Date: %{customdata[0]}",
@@ -603,11 +764,11 @@ class Components:
             "Pct Change: %{customdata[5]}%",
         ])
 
-        self.selection_title(para_dict, para_values)
 
         title = ''
-        for i, key in enumerate(para_dict):
-            title = title + f'{key}:{para_values[i]} '
+        para_dict = para_combination['para_dict']
+        for key in para_dict:
+            title = title + f'{key}:{para_combination[key]} '
 
         # Count number of subchart
         subchart_count = 0
@@ -627,7 +788,7 @@ class Components:
         fig_line = make_subplots(rows=subchart_count+1, cols=1,
                                  row_heights=row_height,shared_xaxes=True)
 
-        fig_line.update_layout(title = {'text': title , 'font': {'size': 10 }  })
+        fig_line.update_layout(title = {'text': title , 'font': {'size': 12 }  })
         fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
         fig_line.update_yaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
         fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
@@ -642,7 +803,7 @@ class Components:
                                xaxis={'title': ''}
                                )
         fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
-                                      x=df_chart['date'], y=df_chart['open'], visible='legendonly',
+                                      x=df_chart['date'], y=df_chart['open'], # visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=9,
                                                   line=dict(color='yellow', width=2.5)), name='open'), row=1, col=1)
         fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
@@ -676,7 +837,7 @@ class Components:
 
 
 
-        return fig_line
+        return df_csv, fig_line
 
 
     def generate_subchart(self,df_chart, element, line_type, line_color):
@@ -751,6 +912,8 @@ class Components:
             if key == 'date':
                 # parameters['date'] = f'{content[:8]} - {content[8:]}'
                 parameters['startdate'], parameters['enddate'] = content[:8], content[8:]
+            elif key == 'summary_mode':
+                pass
             else:
                 parameters[key] = content
 
@@ -810,6 +973,7 @@ class Components:
                     'total_commission': row['total_commission'],
                     'sharp_ratio': row['sharp'],
                 }
+                dict['para_combination'] = ast.literal_eval(row['para_combination'])
                 start_date_year = datetime.datetime.strptime(parameters['startdate'], '%Y%m%d').year
                 end_date_year = datetime.datetime.strptime(parameters['enddate'], '%Y%m%d').year
                 year_list = list(range(start_date_year, end_date_year + 1))
