@@ -14,6 +14,8 @@ class Plot:
     add_button_count = 0
     chart1_button_clicks = 0
     chart2_button_clicks = 0
+    chart3_button_clicks = 0
+    chart3_back_button_clicks = 0
     save_clicks = 0
     line_selected = -1
     button_list = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
@@ -26,8 +28,11 @@ class Plot:
     mdd = 0
     sharp = 0
 
+    start_date_datetime = 0
+    end_date_datetime = 0
+
+    para_combination = {}
     save_path = 'save_path'
-    initial_capital = 0
     net_profit_to_mdd = 0
     net_profit = 0
     mdd_dollar = 0
@@ -42,8 +47,23 @@ class Plot:
     year_win_count = []
 
 
+    def __new__(self, all_para_combination, result_df, settings):
+        # start_date, end_date and para_dict is the same for all combination, ie use the 1st one
+        start_date = all_para_combination[0]['start_date']
+        end_date = all_para_combination[0]['end_date']
+        para_dict = all_para_combination[0]['para_dict']
+        py_filename = all_para_combination[0]['py_filename']
+        output_folder = all_para_combination[0]['output_folder']
+        intraday = all_para_combination[0]['intraday']
+        summary_mode = all_para_combination[0]['summary_mode']
+        freq = all_para_combination[0]['freq']
+        self.initial_capital = all_para_combination[0]['sec_profile']['initial_capital']
+
+        # For chart3 initial date range
+        self.start_date_datetime = datetime.datetime.strptime(start_date, '%Y-%m-%d')
+        self.end_date_datetime = self.start_date_datetime + datetime.timedelta(days=10)
+
 
-    def __new__(self, filename, output_folder, start_date, end_date, para_dict, result_df, generate_filepath, settings):
 
         chart_bg = self.chart_bg
 
@@ -145,23 +165,72 @@ class Plot:
                     # Right Column
                     dbc.Col(html.Div([
                         html.Div(style={'height': '5px', }),
-                        html.Div([html.Div(html.Img(),style={'height':'10px'}),
 
-                                  dbc.Row([
-                                      dbc.Col(html.Div(id='chart1_button', children='',
-                                                       style={'text-align': 'left', 'cursor': 'pointer',
-                                                              'font-size': '14px'}), width=4),
-                                      dbc.Col(html.Div(html.Img()), width=4),
-                                      dbc.Col(html.Div(id='chart2_button', children=''), width=4),
-                                  ], style={'margin': '0px 5px'}),
-                                  html.Div(id='chart_area',
-                                           children=dcc.Graph(id='line_chart', figure=empty_line_chart)),
-                                  html.Div(style={'height': '10px', }),
+                        html.Div(id='main_chart_area',
+                            children=[html.Div(html.Img(),style={'height':'10px'}),
+
+                                      dbc.Row([
+                                          dbc.Col(html.Div(id='chart1_button', children='',
+                                                           style={'text-align': 'left', 'cursor': 'pointer',
+                                                                  'font-size': '14px'}), width=4),
+                                          dbc.Col(html.Div(html.Img()), width=4),
+                                          dbc.Col(html.Div(id='chart2_button', children=''), width=4),
+                                      ], style={'margin': '0px 5px'}),
+                                      html.Div(id='chart_area',
+                                               children=dcc.Graph(id='line_chart', figure=empty_line_chart)),
+
+                                      html.Div(style={'height': '10px', }),
+
+                                     ],style={'padding':'5px','border-radius':'5px','background-color':chart_bg}),
+
+
+                        html.Div(id='chart3_area',
+                                 children=[html.Div(html.Img(), style={'height': '10px'}),
+
+                                           dbc.Row([
+                                               dbc.Col(html.Div(id='chart3_back_button', children='< Back',
+                                                                style={'text-align': 'left', 'cursor': 'pointer',
+                                                                       'font-size': '14px'}), width=4),
+                                               dbc.Col(html.Div(html.Img()), width=4)
+
+                                           ], style={'margin': '0px 5px'}),
+                                           html.Div(id='chart3',
+                                                    children=dcc.Graph(id='chart3_chart', figure=empty_line_chart)),
+
+                                           html.Div(style={'height': '10px', }),
+
+                                           ],
+                                 style={'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg, 'display': 'none'}),
+
+                        html.Div(id='chart3_dashboard',
+                            children=[html.Div(style={'height': '5px', }),
+                                  html.Div(children=dbc.Row([
+                                                dbc.Col(width=1),
+                                                dbc.Col(children=html.Div([
+                                                    html.Div(style={'height': '0.5px'}),
+                                                    dcc.DatePickerRange(
+                                                        id='chart3_date',
+                                                        style={'margin-left':'15px'},
+                                                        month_format='MMM Do, YY',
+                                                        end_date_placeholder_text='MMM Do, YY',
+                                                        start_date=datetime.date(self.start_date_datetime.year, self.start_date_datetime.month, self.start_date_datetime.day),
+                                                        end_date=datetime.date(self.end_date_datetime.year, self.end_date_datetime.month, self.end_date_datetime.day),)
+                                                ]),width=6),
+                                                dbc.Col(children=html.Div(id='chart3_button',
+                                                                          children=html.Div([html.Div(style={'height': '11px'}),'Show Intra Day Candlestick Chart']),
+                                                                          style={'margin': '0px', 'width': '300px', 'backgroundColor': 'blue', 'padding': '0px',
+                                                                                    'border-radius': '2.5px', 'text-align': 'center', 'cursor': 'pointer',
+                                                                                    'font-size': '18px', 'height':'48px'}
+                                                                    ), width=3),
+                                            ], style={'width':'650px'})
+                                           ,style={'padding': '15px 5px', 'border-radius': '5px', 'background-color': chart_bg})
+                                  ], style={'display': 'none'}),
 
-                                 ],style={'padding':'5px','border-radius':'5px','background-color':chart_bg}),
                         html.Div(style={'height': '5px', }),
+
                         html.Div(id='hist_area', children=html.Div(),
                                  style={'display': 'none'}),
+
                     ]), style={'padding':'0'}, width=8),
                 ])
             ),
@@ -180,6 +249,12 @@ class Plot:
             Output('chart2_button', 'children'),
             Output('hist_area', 'children'),
             Output('hist_area', 'style'),
+            Output('main_chart_area', 'style'),
+            Output('chart3_dashboard', 'style'),
+            Output('chart3_area', 'style'),
+            Output('chart3_date', 'start_date'),
+            Output('chart3_date', 'end_date'),
+            Output('chart3_chart', 'figure'),
             Input('sort_method', 'value'),
             Input({'type': 'para-checklist', 'index': ALL}, 'value'),
             State('filter_dropdown_div', 'children'),
@@ -189,24 +264,74 @@ class Plot:
             Input('add_button', 'n_clicks'),
             Input('chart1_button', 'n_clicks'),
             Input('chart2_button', 'n_clicks'),
+            Input('chart3_button', 'n_clicks'),
+            Input('chart3_back_button', 'n_clicks'),
             State('add_button', 'style'),
             State('filter_name', 'value'),
             State('filter_input', 'value'),
             State('hist_area', 'children'),
             State('hist_area', 'style'),
+            State('main_chart_area', 'style'),
+            State('chart3_dashboard', 'style'),
+            State('chart3_area', 'style'),
+            State('chart3_date', 'start_date'),
+            State('chart3_date', 'end_date'),
+            State('chart3_chart', 'figure'),
             [Input('button_' + str(i), 'n_clicks') for i in range(10)],
         )
         def display_output(sort_method,para_checklist,filter_dropdown_div,filter_input_div,fig_line,\
                            filter_div,add_button_clicks, \
-                           chart1_button_clicks,chart2_button_clicks,\
+                           chart1_button_clicks,chart2_button_clicks, chart3_button_clicks, chart3_back_button_clicks, \
                            _add_button_style,filter_name,filter_input_value,\
-                           hist_area,hist_style, *vals):
+                           hist_area,hist_style,main_chart_area_style,chart3_dashboard_style, chart3_area_style, \
+                           chart3_start_str, chart3_end_str, chart3_chart, *vals):
 
             chart1_button_text = ''
             chart2_button_text = ''
 
             chart1_button_click = False
 
+            if chart3_button_clicks:
+                if chart3_button_clicks > self.chart3_button_clicks:
+                    self.chart3_button_clicks = chart3_button_clicks
+
+                    if (chart3_start_str and chart3_end_str):
+                        chart3_start = datetime.datetime.strptime(chart3_start_str, '%Y-%m-%d')
+                        chart3_end = datetime.datetime.strptime(chart3_end_str, '%Y-%m-%d')
+                        detla = chart3_end - chart3_start
+
+                        if (detla.days > 60) or (detla.days < 1):
+                            print('CandleStick Chart Limit: 1 to 60 Days')
+                        else:
+                            para_combination = self.para_combination
+                            hist_style = {'display': 'none'}
+
+                            chart3_chart = components.generate_chart_3(para_combination, chart3_start, chart3_end, freq)
+
+                            main_chart_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg, 'display': 'none'}
+                            chart3_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,}
+
+
+                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
+                           None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
+                           main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
+                           chart3_start_str, chart3_end_str, chart3_chart
+
+            if chart3_back_button_clicks:
+                if chart3_back_button_clicks > self.chart3_back_button_clicks:
+                    self.chart3_back_button_clicks = chart3_back_button_clicks
+                    chart1_button_text = '< Back'
+                    hist_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}
+                    main_chart_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,}
+                    chart3_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,
+                                         'display': 'none'}
+
+
+                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
+                           None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
+                           main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
+                           chart3_start_str, chart3_end_str, chart3_chart
+
 
             if chart1_button_clicks:
                 if chart1_button_clicks > self.chart1_button_clicks:
@@ -214,6 +339,7 @@ class Plot:
                     self.chart1_button_clicks = chart1_button_clicks
                     self. chart_type = 1
                     hist_style = {'display': 'none'}
+                    chart3_dashboard_style = {'display': 'none'}
 
             if chart2_button_clicks:
                 if chart2_button_clicks > self.chart2_button_clicks:
@@ -221,25 +347,36 @@ class Plot:
                     if self.chart_type == 2:
                         chart1_button_text = '< Back'
                         return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
-                               None, chart1_button_text, chart2_button_text, hist_area, hist_style
+                               None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
+                               main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
+                               chart3_start_str, chart3_end_str, chart3_chart
                     else:
                         if self.line_selected > -1:
-                            para_key_list = list(para_dict)
-                            para_values = []
-                            for key in para_key_list:
-                                para_values.append(self.graph_df.iloc[self.line_selected][key])
-                            # save_path = generate_filepath(filename, output_folder, start_date, end_date, para_dict,
-                            #                               para_values)
+                            para_combination = self.para_combination
                             line_colour = self.graph_df.iloc[self.line_selected].line_colour
-                            save_path = self.save_path
-                            df = pd.read_csv(save_path)
-                            df_chart = components.prepare_df_chart(df)
+
+                            if (intraday or summary_mode):
+                                df, fig_line = components.generate_chart_2_summary(para_combination, line_colour)
+
+                            else:
+                                df, fig_line = components.generate_chart_2_full(para_combination, line_colour, settings)
 
                             self.chart_type = 2
                             chart1_button_text = '< Back'
 
-                            fig_line = components.generate_chart_2(df_chart, line_colour,
-                                                                   para_dict,para_values, settings)
+
+                            # For Chart 3 date range
+                            if intraday:
+                                chart3_dashboard_style = {}
+
+                                df_chart3 = df.loc[df['action'].isnull() == False].copy()
+                                self.start_date_datetime = df_chart3.iloc[0].date - datetime.timedelta(days=1)
+                                self.end_date_datetime = self.start_date_datetime + datetime.timedelta(days=10)
+
+                                chart3_start_str = datetime.date(self.start_date_datetime.year, self.start_date_datetime.month,
+                                              self.start_date_datetime.day)
+                                chart3_end_str = datetime.date(self.end_date_datetime.year, self.end_date_datetime.month,
+                                              self.end_date_datetime.day)
 
 
                             # Histograms
@@ -280,7 +417,9 @@ class Plot:
                             hist_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}
 
                             return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style,\
-                                   None, chart1_button_text, chart2_button_text, hist_area, hist_style
+                                   None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
+                                   main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
+                                   chart3_start_str, chart3_end_str, chart3_chart
 
 
                         else:
@@ -295,6 +434,8 @@ class Plot:
                         self.filter_list = []
                         self.add_button_count = 0
                         self.chart2_button_clicks = 0
+                        self.chart3_button_clicks = 0
+                        self.chart3_back_button_clicks = 0
                         self.save_clicks = 0
                         self.chart_type = 1
                         self.line_selected = -1
@@ -361,35 +502,49 @@ class Plot:
                         else:
                             current_df = current_df.loc[current_df[element[0]] > float(element[2])]
 
+
+
                 current_df = current_df.reset_index(drop=True)
 
 
+
+
                 para_key_list = list(para_dict)
 
+
+
                 if len(para_checklist) > 0:
+
                     and_list = []
                     for i in range(len(para_key_list)):
                         key = para_key_list[i]
                         or_list = []
+
                         for element in para_checklist[i]: # -1 because code / para_dict index
+
                             if element == 'True':element = True
                             elif element == 'False':element = False
+                            # if key == 'tolerance':
+
                             _list = current_df[key] == element
+                            # print(_list)
                             or_list.append(_list)
                         or_list = np.logical_or.reduce(or_list)
+                        # print(key, or_list)
                         and_list.append(or_list)
                     and_list = np.logical_and.reduce(and_list)
+
                     df_checked = current_df.loc[pd.DataFrame(and_list, columns=['check'])['check']].reset_index(
                         drop=True).copy()
                 else:
+
                     df_checked = current_df.copy()
 
                 # Sort
                 graph_df = components.sort_method_df(sort_method, df_checked)
                 self.graph_df = graph_df
 
-                fig_line = components.generate_chart_1(graph_df, para_key_list, generate_filepath, filename,
-                                                      output_folder, start_date, end_date, para_dict)
+                fig_line = components.generate_chart_1(graph_df, all_para_combination, intraday, summary_mode)
 
                 # Disable Chart 2
                 chart2_button_text = 'Strategy Analysis >'
@@ -398,7 +553,9 @@ class Plot:
 
 
             return fig_line, filter_div, filter_dropdown_div,filter_input_div,_add_button_style, \
-                   None, chart1_button_text, chart2_button_text, hist_area, hist_style
+                   None, chart1_button_text, chart2_button_text, hist_area, hist_style, \
+                   main_chart_area_style, chart3_dashboard_style, chart3_area_style, \
+                   chart3_start_str, chart3_end_str, chart3_chart
 
 
 
@@ -420,18 +577,18 @@ class Plot:
                     self.line_selected = i
                     button_style = {'text-align': 'right','cursor': 'pointer','font-size':'14px'}
 
-                    para_key_list = list(para_dict)
-                    para_values = []
-                    for key in para_key_list:
-                        para_values.append(self.graph_df.iloc[self.line_selected][key])
-                    save_path = generate_filepath(filename, output_folder, start_date, end_date, para_dict,
-                                                  para_values)
+                    reference_index = self.graph_df.iloc[i].reference_index
+                    df_all_para_combination = pd.DataFrame(all_para_combination)
+                    para_combination = \
+                    df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict(
+                        'records')[0]
+                    self.para_combination = para_combination
+                    save_path = plotguy.generate_filepath(para_combination=para_combination)
 
                     self.save_clicks = 0 # Needed for save button click tracking
 
                     df = self.graph_df.iloc[i].copy()
                     self.save_path = save_path
-                    self.initial_capital = pd.read_csv(save_path).iloc[0].equity_value
                     self.net_profit_to_mdd = df['net_profit_to_mdd']
                     self.net_profit = df['net_profit']
                     self.mdd_dollar = df['mdd_dollar']
@@ -472,6 +629,7 @@ class Plot:
                 for year in self.year_list:
                     columns.append(year)
                     columns.append(f'{year}_win_count')
+                columns.append('para_combination')
                 pd.DataFrame(columns=columns).to_csv('saved_strategies.csv', index=False)
             else:
                 pass
@@ -509,6 +667,7 @@ class Plot:
 
                             if not save_path in list(saved_df['path']):
                                 df_dict = {'path': save_path,
+                                           'para_combination':str(self.para_combination),
                                            'initial': self.initial_capital,
                                            'net_profit_to_mdd': self.net_profit_to_mdd,
                                            'net_profit': self.net_profit,
