@@ -17,8 +17,35 @@ class Signals:
 
         app = Dash(__name__, external_stylesheets=[dbc.themes.SUPERHERO], suppress_callback_exceptions=True)
 
+        my_css_data = """body {
+                  background-color: #1a2245;
+                }
+                        /* width */
+                ::-webkit-scrollbar {
+                  width: 10px !important;
+                  display: block !important;
+                }
+
+                /* Track */
+                ::-webkit-scrollbar-track {
+                  background: #1f2c56 !important;
+                  border-radius: 10px !important;
+                  display: block !important;
+                }
+
+
+                /* Handle */
+                ::-webkit-scrollbar-thumb {
+                  background: #154360;
+                  border-radius: 10px;
+                }
+                """
+        innerHtmlText = "<style>%s</style>" % my_css_data
+
         app.layout = html.Div([
 
+            dash_dangerously_set_inner_html.DangerouslySetInnerHTML(innerHtmlText),
+
             html.Div(style={'height': '5px', }),
 
             html.Div(
